-- @author Stephan Hennig <stephanhennig@arcor.de>
-- @release v0.2, 2010-02-27
-- @copyright 2009, 2010, Stephan Hennig
-- pattern.lua, auxillary pattern related functions
--
--Copyright (C) 2009, 2010 Stephan Hennig
--
--This file is part of the pattern statistics bundle.
--
--The pattern statistics bundle is free software; you can redistribute
--it and/or modify it under the terms of the GNU General Public License
--as published by the Free Software Foundation; either version 3
--of the License, or (at your option) any later version.
--
--The pattern statistics bundle is distributed in the hope that it will
--be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
--of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--GNU General Public License for more details.
--
--You should have received a copy of the GNU General Public License
--along with this program; if not, see <http://www.gnu.org/licenses/>.




--- Build a simple trie from a pattern file.  Input format is a file with
-- exactly one pattern per line (nothing else).
-- @param file  Pattern file.
-- @return A simple trie.
-- The value associated with an identified pattern is the pattern itself.
function buildSimpleTrieFromPatternFile(file)
  local trie = newTrieNode()
  io.input(file)
  for line in io.lines() do
    local s = string.gsub(line, "%d", "") -- pattern without weights
    insertStringSingle(trie, s, line)
  end
  return trie
end


--- Calculate the effectiveness of a trie (number of matches per pattern)
-- against a word list file.  This function may require huge amounts of
-- memory and run for a long time.
-- @param trie  A pattern trie.
-- @param wfile  A word list file.
-- @return A table with pattern statistics.  Keys are patterns and
-- values are tables of the form { count = /number/, words = /array/ }.
-- Key count contains the total number of times a pattern was identified
-- in the word list file.  Key words is an array with all matching words
-- as keys (and value true).
function calcPatternStatistics(trie, wfile)
  local patternstat = {}
  io.input(wfile)
  for line in io.lines() do
    local patterns = decomposeWord2(trie, line)
    for _, v in pairs(patterns) do
      if patternstat[v] then
        patternstat[v].count = patternstat[v].count + 1
        patternstat[v].words[line] = true
      else
        patternstat[v] = { count = 1, words = { [line] = true } }
      end
    end
  end
  return patternstat
end


--- Output pattern statistics.
-- @param patternstat  A table with pattern statistics as provided by
-- function calcPatternStatistics().
-- @param wordlimit  Maximum number of words to output per pattern.
function showPatternStatistics(patternstat, wordlimit)
  local h = {}
  local pattern_tot = 0
  local k = 0
  for pat, stat in pairs(patternstat) do
    k = k + 1
    h[k] = pat
    pattern_tot = pattern_tot + stat.count
  end
  table.sort(h, function(a, b) return (patternstat[a].count < patternstat[b].count) end)
  local pattern_cum = 0
  for _, pat in pairs(h) do
    local stat = patternstat[pat]
    pattern_cum = pattern_cum + stat.count
    io.write(string.format("%-13s %8d %8.4f %8d %8.4f", pat, stat.count, stat.count*100/pattern_tot, pattern_cum, pattern_cum*100/pattern_tot))
    local i = 0
    for word, _ in pairs(stat.words) do
      i = i + 1
      if i > wordlimit then break end
      io.write(" ", word)
    end
    io.write("\n")
  end
end
